const Mongorito = require('mongorito');
const genForeignName = require('../utils/foreign-name');

/**
 * Model extends Mongorito's Model class with mapping
 * document properties to local properties.
 */
class Model extends Mongorito.Model {
    constructor(props = {}) {
        super(props);

        this._mapProps(props);
    }

    _mapProps(props) {
        for (const key in props) {
            Object.defineProperty(this, key, {
                enumerable: true,
                get() {
                    return this.get(key);
                },
                set(value) {
                    return this.set(key, value);
                }
            });
        }
    }

    hasOne(Model, foreign_key, local_key = '_id') {
        return Model.where(
            foreign_key || genForeignName(Model.name),
            this[local_key]
        ).findOne();
    }

    hasMany(Model, foreign_key, local_key = '_id') {
        return Model.where(
            foreign_key || genForeignName(Model.name),
            this[local_key]
        ).find();
    }

    belongsTo(Model, local_key, foreign_key = '_id') {
        return Model.where(
            foreign_key,
            this[local_key || genForeignName(Model.name)]
        ).findOne();
    }

    belongsToMany(Model, local_key, foreign_key = '_id') {
        return Model.where(
            foreign_key,
            this[local_key || genForeignName(Model.name)]
        ).find();
    }
}

module.exports = Model;
