const express = require('express');
const { controllerify } = require('../controller/index');

/**
 * Clode's Router featuring groups.
 */
class Router {
    constructor(router = express.Router()) {
        if (!router || router.name !== 'router')
            throw new Error("No express's Router instance provided");
        this.router = router;
    }

    async get() {
        const args = await this._controllerify(arguments);
        this.router.get(...args);

        return this;
    }

    async post() {
        const args = await this._controllerify(arguments);
        this.router.post(...args);

        return this;
    }

    async patch() {
        const args = await this._controllerify(arguments);
        this.router.patch(...args);

        return this;
    }

    async put() {
        const args = await this._controllerify(arguments);
        this.router.put(...args);

        return this;
    }

    async delete() {
        const args = await this._controllerify(arguments);
        this.router.delete(...args);

        return this;
    }

    async options() {
        const args = await this._controllerify(arguments);
        this.router.options(...args);

        return this;
    }

    group() {
        const instance = new Router();
        const prefix = arguments[0];
        const callback = arguments[2];
        let middleware = arguments[1];

        if (!Array.isArray(middleware)) {
            middleware = [middleware];
        }

        middleware.forEach(item => {
            instance.router.use(item);
        });

        this.router.use(prefix, instance.router);

        // Return group instance if no callback provided
        if (!callback) return instance;

        // Bing group instance as this and pass it as an argument to callback
        callback.call(instance, instance);
        return this;
    }

    async _controllerify(props) {
        if (props[2]) props[2] = controllerify(props[2]);
        else props[1] = controllerify(props[1]);

        return props;
    }
}

module.exports = Router;
