const chai = require('chai');
const Router = require('../router');
const { middleware } = require('../utils/fakes');

const { expect } = chai;

describe('Router', function() {
    it('Should throw error if router not passed', function(done) {
        expect(() => {
            new Router({});
        }).to.throw("No express's Router instance provided");
        done();
    });

    it('Should create instance', function(done) {
        expect(() => {
            new Router();
        }).to.not.throw();
        done();
    });
});

describe('Router methods', function() {
    const AppRouter = new Router();
    const controller = () => ({ status: 'ok' });

    it('Should add GET and return chain', function(done) {
        (async () => {
            const route = await AppRouter.get('/', controller);
            expect(route).to.be.instanceOf(Router);
            done();
        })();
    });

    it('Should add POST and return chain', function(done) {
        (async () => {
            const route = await AppRouter.post('/', controller);
            expect(route).to.be.instanceOf(Router);
            done();
        })();
    });

    it('Should add PATCH and return chain', function(done) {
        (async () => {
            const route = await AppRouter.patch('/', controller);
            expect(route).to.be.instanceOf(Router);
            done();
        })();
    });

    it('Should add DELETE and return chain', function(done) {
        (async () => {
            const route = await AppRouter.delete('/', controller);
            expect(route).to.be.instanceOf(Router);
            done();
        })();
    });

    it('Should add PUT and return chain', function(done) {
        (async () => {
            const route = await AppRouter.put('/', controller);
            expect(route).to.be.instanceOf(Router);
            done();
        })();
    });

    it('Should add OPTIONS and return chain', function(done) {
        (async () => {
            const route = await AppRouter.options('/', controller);
            expect(route).to.be.instanceOf(Router);
            done();
        })();
    });

    it('Should create group and use callback with group instance as this', function(done) {
        AppRouter.group('/test', [], function() {
            expect(this).to.be.instanceOf(Router);
            done();
        });
    });

    it('Should create group and use callback with group instance as parameter', function(done) {
        AppRouter.group('/test', [], function(test) {
            expect(test).to.be.instanceOf(Router);
            done();
        });
    });

    it('Should create group and return group instance if callback is not provided', function(done) {
        const group = AppRouter.group('/test', []);
        expect(group).to.be.instanceOf(Router);
        done();
    });

    it('Should apply array of middleware', function(done) {
        const group = AppRouter.group('/test', [middleware, middleware]);
        expect(group).to.be.instanceOf(Router);
        done();
    });

    it('Should apply single middleware', function(done) {
        const group = AppRouter.group('/test', middleware);
        expect(group).to.be.instanceOf(Router);
        done();
    });
});

describe('Fake middleware', function() {
    it('Should call next()', function(done) {
        middleware(null, null, () => {
            done();
        });
    });
});
